@@ -18,9 +18,14 @@
 
 use crate::{returnable::RouteContext, Response};
 
-pub type RouteResponse = fn(RouteContext<'_>) -> Response<'_>;
-pub type ErrorResponse =
-  fn(crate::returnable::ErrorContext<'_>) -> Response<'_>;
-pub type Callback =
-  fn(&tokio::net::TcpStream, &url::Url, Option<&matchit::Params<'_, '_>>);
-pub type Partial = fn(RouteContext<'_>) -> String;
+pub type RouteResponse =
+  Box<dyn FnMut(RouteContext<'_>) -> Response<'_> + Send + Sync>;
+pub type ErrorResponse = Box<
+  dyn FnMut(crate::returnable::ErrorContext<'_>) -> Response<'_> + Send + Sync,
+>;
+pub type Callback = Box<
+  dyn FnMut(&tokio::net::TcpStream, &url::Url, Option<&matchit::Params<'_, '_>>)
+    + Send
+    + Sync,
+>;
+pub type Partial = Box<dyn FnMut(RouteContext<'_>) -> String + Send + Sync>;
