@@ -33,15 +33,15 @@
 //! # Cargo.toml
 //!
 //! [dependencies]
-//! windmark = "0.1.11"
+//! windmark = "0.1.13"
 //! tokio = { version = "0.2.4", features = ["full"] }
 //!
 //! # If you would like to use the built-in logger (recommended)
-//! # windmark = { version = "0.1.11", features = ["logger"] }
+//! # windmark = { version = "0.1.13", features = ["logger"] }
 //!
 //! # If you would like to use the built-in MIME dedection when `Success`-ing a file
 //! # (recommended)
-//! # windmark = { version = "0.1.11", features = ["auto-deduce-mime"] }
+//! # windmark = { version = "0.1.13", features = ["auto-deduce-mime"] }
 //! ```
 //!
 //! ### Implement a Windmark server
@@ -107,7 +107,7 @@
 #![deny(clippy::all, clippy::nursery, clippy::pedantic)]
 #![recursion_limit = "128"]
 
-mod handler;
+pub mod handler;
 pub mod module;
 pub mod response;
 pub mod returnable;
@@ -137,6 +137,23 @@ use crate::{
   returnable::{CallbackContext, ErrorContext, RouteContext},
 };
 
+macro_rules! or_error {
+  ($stream:ident, $operation:expr, $error_format:literal) => {
+    match $operation {
+      Ok(u) => u,
+      Err(e) => {
+        $stream
+          .write_all(format!($error_format, e).as_bytes())
+          .await?;
+
+        $stream.shutdown().await?;
+
+        return Ok(());
+      }
+    }
+  };
+}
+
 /// A router which takes care of all tasks a Windmark server should handle:
 /// response generation, panics, logging, and more.
 #[derive(Clone)]
@@ -353,11 +370,18 @@ impl Router {
     let mut header = String::new();
 
     while let Ok(size) = stream.read(&mut buffer).await {
-      let content = String::from_utf8(buffer[0..size].to_vec())?;
+      let request = or_error!(
+        stream,
+        String::from_utf8(buffer[0..size].to_vec()),
+        "59 The server (Windmark) received a bad request: {}"
+      );
+      url = or_error!(
+        stream,
+        url::Url::parse(&request.replace("\r\n", "")),
+        "59 The server (Windmark) received a bad request: {}"
+      );
 
-      url = url::Url::parse(&content.replace("\r\n", ""))?;
-
-      if content.contains("\r\n") {
+      if request.contains("\r\n") {
         break;
       }
     }
@@ -365,7 +389,11 @@ impl Router {
     let fixed_path = if self.fix_path {
       self
         .routes
-        .fix_path(url.path())
+        .fix_path(if url.path().is_empty() {
+          "/"
+        } else {
+          url.path()
+        })
         .unwrap_or_else(|| url.path().to_string())
     } else {
       url.path().to_string()
